@@ -3,7 +3,7 @@ from twisted.trial import unittest
 
 from twisted.internet import protocol, defer, reactor
 from twisted.application import internet
-from foolscap import pb, negotiate, tokens
+from foolscap import pb, negotiate, tokens, eventual
 from foolscap import Referenceable, Tub, UnauthenticatedTub, BananaError
 from foolscap.eventual import flushEventualQueue
 crypto_available = False
@@ -161,6 +161,17 @@ class BaseMixin:
         self.target = Target()
         return tub.registerReference(self.target), l.getPortnum()
 
+    def createSpecificServer(self, certData,
+                             negotiationClass=negotiate.Negotiation):
+        tub = Tub(certData=certData)
+        tub.negotiationClass = negotiationClass
+        tub.startService()
+        self.services.append(tub)
+        l = tub.listenOn("tcp:0")
+        tub.setLocation("127.0.0.1:%d" % l.getPortnum())
+        target = Target()
+        return tub, target, tub.registerReference(target), l.getPortnum()
+
     def makeNullServer(self):
         f = protocol.Factory()
         f.protocol = protocol.Protocol # discards everything
@@ -374,7 +385,7 @@ class Parallel(BaseMixin, unittest.TestCase):
     #     (sent GET, waiting for the 101 Switching Protocols)
     #  4: still in ENCRYPTED phase: sent Hello, waiting for their Hello
     #  5: in DECIDING phase (non-master), waiting for their decision
-    #   
+    #
 
     def makeServers(self, tubopts={}, lo1={}, lo2={}):
         self.requireCrypto()
@@ -751,8 +762,8 @@ MAX_HANDLED_VERSION = negotiate.Negotiation.maxVersion
 UNHANDLED_VERSION = 4
 class NegotiationVbig(negotiate.Negotiation):
     maxVersion = UNHANDLED_VERSION
-    def __init__(self):
-        negotiate.Negotiation.__init__(self)
+    def __init__(self, logparent):
+        negotiate.Negotiation.__init__(self, logparent)
         self.negotiationOffer["extra"] = "new value"
     def evaluateNegotiationVersion4(self, offer):
         # just like v1, but different
@@ -772,7 +783,7 @@ class Future(BaseMixin, unittest.TestCase):
         # negotiation decision
         self.requireCrypto()
         url, portnum = self.makeSpecificServer(certData_high)
-        # the client 
+        # the client
         client = Tub(certData=certData_low)
         client.negotiationClass = NegotiationVbig
         client.startService()
@@ -790,7 +801,7 @@ class Future(BaseMixin, unittest.TestCase):
         # and thus make the negotiation decision
         self.requireCrypto()
         url, portnum = self.makeSpecificServer(certData_low)
-        # the client 
+        # the client
         client = Tub(certData=certData_high)
         client.negotiationClass = NegotiationVbig
         client.startService()
@@ -824,7 +835,7 @@ class Future(BaseMixin, unittest.TestCase):
         # understands [1,2]
         self.requireCrypto()
         url, portnum = self.makeSpecificServer(certData_low, NegotiationVbig)
-        # the client 
+        # the client
         client = Tub(certData=certData_high)
         client.startService()
         self.services.append(client)
@@ -844,7 +855,7 @@ class Future(BaseMixin, unittest.TestCase):
         # negotiation decision
         self.requireCrypto()
         url, portnum = self.makeSpecificServer(certData_high)
-        # the client 
+        # the client
         client = Tub(certData=certData_low)
         client.negotiationClass = NegotiationVbigOnly
         client.startService()
@@ -912,6 +923,307 @@ class Future(BaseMixin, unittest.TestCase):
         return d
     testTooFarInFuture4.timeout = 10
 
+
+class Replacement(BaseMixin, unittest.TestCase):
+    # in certain circumstances, a new connection is supposed to replace an
+    # existing one.
+
+    def createDuplicateServer(self, oldtub):
+        tub = Tub(certData=oldtub.getCertData())
+        tub.startService()
+        self.services.append(tub)
+        tub.incarnation = oldtub.incarnation
+        tub.incarnation_string = oldtub.incarnation_string
+        tub.slave_table = oldtub.slave_table.copy()
+        tub.master_table = oldtub.master_table.copy()
+        l = tub.listenOn("tcp:0")
+        tub.setLocation("127.0.0.1:%d" % l.getPortnum())
+        target = Target()
+        return tub, target, tub.registerReference(target), l.getPortnum()
+
+    def setUp(self):
+        BaseMixin.setUp(self)
+        self.requireCrypto()
+        (self.tub1, self.target1, self.furl1, l1) = \
+                    self.createSpecificServer(certData_low)
+        (self.tub2, self.target2, self.furl2, l2) = \
+                    self.createSpecificServer(certData_high)
+        # self.tub1 is the slave, self.tub2 is the master
+        assert self.tub2.tubID > self.tub1.tubID
+
+    def clone_servers(self):
+        (self.tub1a, self.target1a, self.furl1a, l1a) = \
+                     self.createDuplicateServer(self.tub1)
+        (self.tub2a, self.target2a, self.furl2a, l2a) = \
+                     self.createDuplicateServer(self.tub2)
+
+    def testBouncedClient(self):
+        # self.tub1 is the slave, self.tub2 is the master
+
+        d = self.tub1.getReference(self.furl2)
+        d2 = defer.Deferred()
+        def _connected(rref):
+            self.clone_servers()
+            # our tub1a is not the same incarnation as tub1
+            self.tub1a.make_incarnation()
+
+            # a new incarnation of the slave should replace the old connection
+            rref.notifyOnDisconnect(d2.callback, None)
+            return self.tub1a.getReference(self.furl2)
+        d.addCallback(_connected)
+        # the old rref should be broken (eventually)
+        d.addCallback(lambda res: d2)
+        return d
+
+    def shouldFail(self, expected_failure, which, substring=None,
+                   func=None, *args, **kwargs):
+        assert func
+        d = defer.maybeDeferred(func, *args, **kwargs)
+        def _pass(res):
+            self.fail("expected to see failure (%s), not success (%s), at %s"
+                      % (expected_failure, res, which))
+        def _fail(f):
+            f.trap(expected_failure)
+            if substring:
+                self.failUnless(substring in str(f),
+                                "substring '%s' not in '%s' at %s" %
+                                (substring, str(f), which))
+        d.addCallbacks(_pass, _fail)
+        return d
+
+    def testAncientClient(self):
+        disconnects = []
+        d = self.tub1.getReference(self.furl2)
+        def _connected(rref):
+            self.clone_servers()
+            # old clients (foolscap-0.1.7 or earlier) don't send a
+            # my-incarnation header, so we're supposed to reject their
+            # connection offer
+            self.tub1a.incarnation_string = ""
+
+            # this new connection attempt will be rejected
+            rref.notifyOnDisconnect(disconnects.append, 1)
+            return self.shouldFail(tokens.RemoteNegotiationError,
+                                   "testAncientClient",
+                                   "Duplicate connection",
+                                   self.tub1a.getReference, self.furl2)
+        d.addCallback(_connected)
+        def _stall(res):
+            return eventual.fireEventually(res)
+        d.addCallback(_stall)
+        def _check(res):
+            self.failIf(disconnects)
+        d.addCallback(_check)
+        return d
+
+    def testAncientClientWorkaround(self):
+        self.tub2.setOption("handle-old-duplicate-connections", True)
+        # the second connection will be dropped, because it shows up too
+        # quickly.
+        disconnects = []
+        d2 = defer.Deferred()
+        d = self.tub1.getReference(self.furl2)
+        def _connected(rref):
+            self.clone_servers()
+            # old clients (foolscap-0.1.7 or earlier) don't send a
+            # my-incarnation header, so we're supposed to reject their
+            # connection offer
+            self.tub1a.incarnation_string = ""
+
+            # this new connection attempt will be rejected
+            rref.notifyOnDisconnect(disconnects.append, 1)
+            rref.notifyOnDisconnect(d2.callback, None)
+            return self.shouldFail(tokens.RemoteNegotiationError,
+                                   "testAncientClientWorkaround",
+                                   "Duplicate connection",
+                                   self.tub1a.getReference, self.furl2)
+        d.addCallback(_connected)
+        def _stall(res):
+            return eventual.fireEventually(res)
+        d.addCallback(_stall)
+        def _check(res):
+            self.failIf(disconnects)
+        d.addCallback(_check)
+
+        # now we tweak the connection-is-old threshold to allow the third
+        # connection to succeed.
+        def _reconnect(rref):
+            self.tub2._handle_old_duplicate_connections = -10
+            return self.tub1a.getReference(self.furl2)
+        d.addCallback(_reconnect)
+        # the old rref should be broken (eventually)
+        d.addCallback(lambda res: d2)
+
+        return d
+
+
+    def testLostDecisionMessage_NewServer(self):
+        # doctor the client's memory, make it think that it had a connection
+        # to a different incarnation of the server
+
+        # this test exercises the offer_master_IR != self.tub..IR case
+
+        d = self.tub1.getReference(self.furl2)
+        d2 = defer.Deferred()
+        def _connected(rref):
+            # if the slave thinks it was connected to an earlier master, we
+            # accept the new connection
+            self.clone_servers()
+            oldrecord = self.tub1.slave_table[self.tub2.tubID]
+            self.tub1a.slave_table[self.tub2.tubID] = ("figment", oldrecord[1])
+            rref.notifyOnDisconnect(d2.callback, None)
+            return self.tub1a.getReference(self.furl2)
+        d.addCallback(_connected)
+        # the old rref should be broken (eventually)
+        d.addCallback(lambda res: d2)
+        return d
+
+    def testTwoLostDecisionMessages(self):
+        # the client connects successfully with seqnum=1. Then the client
+        # thinks the connection is lost, so it tries to reconnect, the server
+        # accepts (seqnum=2), but the decision message gets lost. Then the
+        # client tries to connect a third time: the client says it knows
+        # about seqnum=1, which is older than the current one. We should
+        # reject the third attempt.
+
+        # we represent this case by connecting once, disconnecting,
+        # reconnecting, then having the second tub connect with an
+        # artificially-decremented seqnum.
+
+        # this test exercises the offer_master_seqnum < existing_seqnum case
+
+        disconnects = []
+        d = self.tub1.getReference(self.furl2)
+        def _connect1(rref):
+            d2 = defer.Deferred()
+            rref.notifyOnDisconnect(d2.callback, None)
+            rref.tracker.broker.transport.loseConnection()
+            return d2
+        d.addCallback(_connect1)
+        def _reconnect(res):
+            return self.tub1.getReference(self.furl2)
+        d.addCallback(_reconnect)
+        def _connect2(rref):
+            self.clone_servers()
+            old_record = self.tub1a.slave_table[self.tub2.tubID]
+            (old_IR, old_seqnum) = old_record
+            new_record = (old_IR, str(int(old_seqnum)-1))
+            self.tub1a.slave_table[self.tub2.tubID] = new_record
+
+            # this new connection attempt will be rejected
+            rref.notifyOnDisconnect(disconnects.append, 1)
+            return self.shouldFail(tokens.RemoteNegotiationError,
+                                   "testTwoLostDecisionMessages",
+                                   "Duplicate connection",
+                                   self.tub1a.getReference, self.furl2)
+        d.addCallback(_connect2)
+        def _stall(res):
+            return eventual.fireEventually(res)
+        d.addCallback(_stall)
+        def _check(res):
+            self.failIf(disconnects)
+        d.addCallback(_check)
+        return d
+
+    def testWeirdSeqnum(self):
+        # if the client sends a seqnum that's too far into the future,
+        # something weird is going on, and we should reject the offer.
+
+        # this test exercises the offer_master_seqnum > existing_seqnum case
+
+        disconnects = []
+        d = self.tub1.getReference(self.furl2)
+        def _connected(rref):
+            self.clone_servers()
+            old_record = self.tub1a.slave_table[self.tub2.tubID]
+            (old_IR, old_seqnum) = old_record
+            new_record = (old_IR, str(int(old_seqnum)+10))
+            self.tub1a.slave_table[self.tub2.tubID] = new_record
+            # this new connection attempt will be rejected
+            rref.notifyOnDisconnect(disconnects.append, 1)
+            return self.shouldFail(tokens.RemoteNegotiationError,
+                                   "testSimultaneousClient",
+                                   "Duplicate connection",
+                                   self.tub1a.getReference, self.furl2)
+        d.addCallback(_connected)
+        def _stall(res):
+            return eventual.fireEventually(res)
+        d.addCallback(_stall)
+        def _check(res):
+            self.failIf(disconnects)
+        d.addCallback(_check)
+        return d
+
+    def testNATEntryDropped(self):
+        # a client connects successfully, and receives the decision, but then
+        # the connection goes away such that the client sees it but the
+        # server does not. The new connection should be accepted.
+
+        # this test exercises the offer_master_seqnum == existing_seqnum case
+
+        d = self.tub1.getReference(self.furl2)
+        d2 = defer.Deferred()
+        def _connected(rref):
+            self.clone_servers()
+            # leave the slave_table entry intact
+            rref.notifyOnDisconnect(d2.callback, None)
+            return self.tub1a.getReference(self.furl2)
+        d.addCallback(_connected)
+        # the old rref should be broken (eventually)
+        d.addCallback(lambda res: d2)
+        return d
+
+    def testConnectionHintRace(self):
+        # doctor the client to make the second connection look like it came
+        # from the same batch as the existing one. This should be rejected:
+        # this is the multiple-connection-hints case.
+
+        # This is also what happens when a decision message is droped.
+
+        # since this is the first time the slave tried to connect, this test
+        # exercises the offer_master_IR == "none" case
+
+        disconnects = []
+        d = self.tub1.getReference(self.furl2)
+        def _connected(rref):
+            self.clone_servers()
+            del self.tub1a.slave_table[self.tub2.tubID]
+            # this new connection attempt will be rejected
+            rref.notifyOnDisconnect(disconnects.append, 1)
+            return self.shouldFail(tokens.RemoteNegotiationError,
+                                   "testSimultaneousClient",
+                                   "Duplicate connection",
+                                   self.tub1a.getReference, self.furl2)
+        d.addCallback(_connected)
+        def _stall(res):
+            return eventual.fireEventually(res)
+        d.addCallback(_stall)
+        def _check(res):
+            self.failIf(disconnects)
+        d.addCallback(_check)
+        return d
+
+    def testBouncedClient_Reverse(self):
+        # self.tub1 is the master, self.tub2 is the slave
+
+        d = self.tub2.getReference(self.furl1)
+        d2 = defer.Deferred()
+        def _connected(rref):
+            self.clone_servers()
+            # our tub2a is not the same incarnation as tub2
+            self.tub2a.make_incarnation()
+
+            # a new incarnation of the master should replace the old connection
+            rref.notifyOnDisconnect(d2.callback, None)
+            return self.tub2a.getReference(self.furl1)
+        d.addCallback(_connected)
+        # the old rref should be broken (eventually)
+        d.addCallback(lambda res: d2)
+        return d
+
+
+
+
 # disable all tests unless NEWPB_TEST_NEGOTIATION is set in the environment.
 # The negotiation tests are sensitive to system load, and the intermittent
 # failures are really annoying. The 'right' solution to this involves
