@@ -204,7 +204,7 @@ class NewDirectoryNode:
         def _update(children):
             children[name] = (children[name][0], metadata)
             new_contents = self._pack_contents(children)
-            return self._node.replace(new_contents)
+            return self._node.update(new_contents)
         d.addCallback(_update)
         d.addCallback(lambda res: self)
         return d
@@ -250,7 +250,7 @@ class NewDirectoryNode:
         assert isinstance(name, unicode)
         return self.set_node(name, self._create_node(child_uri), metadata)
 
-    def set_uris(self, entries):
+    def set_children(self, entries):
         node_entries = []
         for e in entries:
             if len(e) == 2:
@@ -306,7 +306,7 @@ class NewDirectoryNode:
                     metadata = new_metadata.copy()
                 children[name] = (child, metadata)
             new_contents = self._pack_contents(children)
-            return self._node.replace(new_contents)
+            return self._node.update(new_contents)
         d.addCallback(_add)
         d.addCallback(lambda res: None)
         return d
@@ -337,7 +337,7 @@ class NewDirectoryNode:
             old_child, metadata = children[name]
             del children[name]
             new_contents = self._pack_contents(children)
-            d = self._node.replace(new_contents)
+            d = self._node.update(new_contents)
             def _done(res):
                 return old_child
             d.addCallback(_done)
