@@ -1,4 +1,5 @@
 import os, re, weakref, stat, struct, time
+from distutils.version import LooseVersion
 from itertools import chain
 
 from foolscap import Referenceable
@@ -677,8 +678,7 @@ class StorageServer(service.MultiService, Referenceable):
     implements(RIStorageServer, IStatsProducer)
     name = 'storage'
 
-    # we're pretty narrow-minded right now
-    OLDEST_SUPPORTED_VERSION = allmydata.__version__
+    OLDEST_SUPPORTED_VERSION = LooseVersion("0.8.0")
 
     def __init__(self, storedir, sizelimit=None,
                  discard_storage=False, readonly_storage=False,
@@ -1201,10 +1201,20 @@ class WriteBucketProxy:
 
 class ReadBucketProxy:
     implements(IStorageBucketReader)
-    def __init__(self, rref):
+    def __init__(self, rref, peerid=None, storage_index_s=None):
         self._rref = rref
+        self._peerid = peerid
+        self._si_s = storage_index_s
         self._started = False
 
+    def get_peerid(self):
+        return self._peerid
+
+    def __repr__(self):
+        peerid_s = idlib.shortnodeid_b2a(self._peerid)
+        return "<ReadBucketProxy to peer [%s] SI %s>" % (peerid_s,
+                                                         self._si_s)
+
     def startIfNecessary(self):
         if self._started:
             return defer.succeed(self)
