@@ -780,6 +780,30 @@ class POSTHandler(rend.Page):
         d.addCallback(lambda res: "directory created")
         return d
 
+    def _POST_mkdir_p(self, path):
+        path_ = tuple([seg.decode("utf-8") for seg in path.split('/') if seg ])
+        d = self._get_or_create_directories(self._node, path_)
+        d.addCallback(lambda node: node.get_uri())
+        return d
+
+    # this code stolen from PUTHandler: should be refactored to a more
+    # generally accesible place, perhaps...
+    def _get_or_create_directories(self, node, path):
+        if not IDirectoryNode.providedBy(node):
+            # unfortunately it is too late to provide the name of the
+            # blocking directory in the error message.
+            raise BlockingFileError("cannot create directory because there "
+                                    "is a file in the way")
+        if not path:
+            return defer.succeed(node)
+        d = node.get(path[0])
+        def _maybe_create(f):
+            f.trap(KeyError)
+            return node.create_empty_directory(path[0])
+        d.addErrback(_maybe_create)
+        d.addCallback(self._get_or_create_directories, path[1:])
+        return d
+
     def _POST_uri(self, name, newuri):
         d = self._check_replacement(name)
         d.addCallback(lambda res: self._node.set_uri(name, newuri))
@@ -843,7 +867,7 @@ class POSTHandler(rend.Page):
                 return d2
             d.addCallback(_checked)
         else:
-            uploadable = FileHandle(contents.file)
+            uploadable = FileHandle(contents.file, convergence=client.convergence)
             d = self._check_replacement(name)
             d.addCallback(lambda res: self._node.add_file(name, uploadable))
             def _done(newnode):
@@ -914,6 +938,11 @@ class POSTHandler(rend.Page):
             if not name:
                 raise RuntimeError("mkdir requires a name")
             d = self._POST_mkdir(name)
+        elif t == "mkdir-p":
+            path = get_arg(req, "path")
+            if not path:
+                raise RuntimeError("mkdir-p requires a path")
+            d = self._POST_mkdir_p(path)
         elif t == "uri":
             if not name:
                 raise RuntimeError("set-uri requires a name")
@@ -1018,6 +1047,7 @@ class PUTHandler(rend.Page):
         self._replace = replace
 
     def renderHTTP(self, ctx):
+        client = IClient(ctx)
         req = inevow.IRequest(ctx)
         t = self._t
         localfile = self._localfile
@@ -1034,18 +1064,18 @@ class PUTHandler(rend.Page):
         d.addCallback(self._check_replacement, name, self._replace)
         if t == "upload":
             if localfile:
-                d.addCallback(self._upload_localfile, localfile, name)
+                d.addCallback(self._upload_localfile, localfile, name, convergence=client.convergence)
             else:
                 # localdir
                 # take the last step
                 d.addCallback(self._get_or_create_directories, self._path[-1:])
-                d.addCallback(self._upload_localdir, localdir)
+                d.addCallback(self._upload_localdir, localdir, convergence=client.convergence)
         elif t == "uri":
             d.addCallback(self._attach_uri, req.content, name)
         elif t == "mkdir":
             d.addCallback(self._mkdir, name)
         else:
-            d.addCallback(self._upload_file, req.content, name)
+            d.addCallback(self._upload_file, req.content, name, convergence=client.convergence)
 
         def _transform_error(f):
             errors = {BlockingFileError: http.BAD_REQUEST,
@@ -1097,8 +1127,8 @@ class PUTHandler(rend.Page):
         d.addCallback(_done)
         return d
 
-    def _upload_file(self, node, contents, name):
-        uploadable = FileHandle(contents)
+    def _upload_file(self, node, contents, name, convergence):
+        uploadable = FileHandle(contents, convergence=convergence)
         d = node.add_file(name, uploadable)
         def _done(filenode):
             log.msg("webish upload complete",
@@ -1107,8 +1137,8 @@ class PUTHandler(rend.Page):
         d.addCallback(_done)
         return d
 
-    def _upload_localfile(self, node, localfile, name):
-        uploadable = FileName(localfile)
+    def _upload_localfile(self, node, localfile, name, convergence):
+        uploadable = FileName(localfile, convergence=convergence)
         d = node.add_file(name, uploadable)
         d.addCallback(lambda filenode: filenode.get_uri())
         return d
@@ -1121,7 +1151,7 @@ class PUTHandler(rend.Page):
         d.addCallback(_done)
         return d
 
-    def _upload_localdir(self, node, localdir):
+    def _upload_localdir(self, node, localdir, convergence):
         # build up a list of files to upload. TODO: for now, these files and
         # directories must have UTF-8 encoded filenames: anything else will
         # cause the upload to break.
@@ -1150,7 +1180,7 @@ class PUTHandler(rend.Page):
             if dir:
                 d.addCallback(self._makedir, node, dir)
         for f in all_files:
-            d.addCallback(self._upload_one_file, node, localdir, f)
+            d.addCallback(self._upload_one_file, node, localdir, f, convergence=convergence)
         return d
 
     def _makedir(self, res, node, dir):
@@ -1162,12 +1192,12 @@ class PUTHandler(rend.Page):
         d.addCallback(lambda parent: parent.create_empty_directory(dir[-1]))
         return d
 
-    def _upload_one_file(self, res, node, localdir, f):
+    def _upload_one_file(self, res, node, localdir, f, convergence):
         # get the parent. We can be sure this exists because we already
         # went through and created all the directories we require.
         localfile = os.path.join(localdir, *f)
         d = node.get_child_at_path(f[:-1])
-        d.addCallback(self._upload_localfile, localfile, f[-1])
+        d.addCallback(self._upload_localfile, localfile, f[-1], convergence=convergence)
         return d
 
 
@@ -1324,67 +1354,69 @@ class Root(rend.Page):
         client = IClient(ctx)
         req = inevow.IRequest(ctx)
 
-        segments = list(segments) # XXX HELP I AM YUCKY!
+        if not segments or segments[0] != "uri":
+            return rend.Page.locateChild(self, ctx, segments)
+
+        segments = list(segments)
         while segments and not segments[-1]:
             segments.pop()
         if not segments:
             segments.append('')
         segments = tuple(segments)
-        if segments:
-            if segments[0] == "uri":
-                if len(segments) == 1 or segments[1] == '':
-                    uri = get_arg(req, "uri", None)
-                    if uri is not None:
-                        there = url.URL.fromContext(ctx)
-                        there = there.clear("uri")
-                        there = there.child("uri").child(uri)
-                        return there, ()
-                if len(segments) == 1:
-                    # /uri
-                    if req.method == "PUT":
-                        # either "PUT /uri" to create an unlinked file, or
-                        # "PUT /uri?t=mkdir" to create an unlinked directory
-                        t = get_arg(req, "t", "").strip()
-                        if t == "":
-                            mutable = bool(get_arg(req, "mutable", "").strip())
-                            if mutable:
-                                return unlinked.UnlinkedPUTSSKUploader(), ()
-                            else:
-                                return unlinked.UnlinkedPUTCHKUploader(), ()
-                        if t == "mkdir":
-                            return unlinked.UnlinkedPUTCreateDirectory(), ()
-                        errmsg = "/uri only accepts PUT and PUT?t=mkdir"
-                        return WebError(http.BAD_REQUEST, errmsg), ()
 
-                    elif req.method == "POST":
-                        # "POST /uri?t=upload&file=newfile" to upload an
-                        # unlinked file or "POST /uri?t=mkdir" to create a
-                        # new directory
-                        t = get_arg(req, "t", "").strip()
-                        if t in ("", "upload"):
-                            mutable = bool(get_arg(req, "mutable", "").strip())
-                            if mutable:
-                                return unlinked.UnlinkedPOSTSSKUploader(), ()
-                            else:
-                                return unlinked.UnlinkedPOSTCHKUploader(client, req), ()
-                        if t == "mkdir":
-                            return unlinked.UnlinkedPOSTCreateDirectory(), ()
-                        errmsg = "/uri accepts only PUT, PUT?t=mkdir, POST?t=upload, and POST?t=mkdir"
-                        return WebError(http.BAD_REQUEST, errmsg), ()
-                if len(segments) < 2:
-                    return rend.NotFound
-                uri = segments[1]
-                d = defer.maybeDeferred(client.create_node_from_uri, uri)
-                d.addCallback(lambda node: VDrive(node, uri))
-                d.addCallback(lambda vd: vd.locateChild(ctx, segments[2:]))
-                def _trap_KeyError(f):
-                    f.trap(KeyError)
-                    return rend.FourOhFour(), ()
-                d.addErrback(_trap_KeyError)
-                return d
-            elif segments[0] == "xmlrpc":
-                raise NotImplementedError()
-        return rend.Page.locateChild(self, ctx, segments)
+        if len(segments) == 1 or segments[1] == '':
+            uri = get_arg(req, "uri", None)
+            if uri is not None:
+                there = url.URL.fromContext(ctx)
+                there = there.clear("uri")
+                there = there.child("uri").child(uri)
+                return there, ()
+
+        if len(segments) == 1:
+            # /uri
+            if req.method == "PUT":
+                # either "PUT /uri" to create an unlinked file, or
+                # "PUT /uri?t=mkdir" to create an unlinked directory
+                t = get_arg(req, "t", "").strip()
+                if t == "":
+                    mutable = bool(get_arg(req, "mutable", "").strip())
+                    if mutable:
+                        return unlinked.UnlinkedPUTSSKUploader(), ()
+                    else:
+                        return unlinked.UnlinkedPUTCHKUploader(), ()
+                if t == "mkdir":
+                    return unlinked.UnlinkedPUTCreateDirectory(), ()
+                errmsg = "/uri only accepts PUT and PUT?t=mkdir"
+                return WebError(http.BAD_REQUEST, errmsg), ()
+
+            elif req.method == "POST":
+                # "POST /uri?t=upload&file=newfile" to upload an
+                # unlinked file or "POST /uri?t=mkdir" to create a
+                # new directory
+                t = get_arg(req, "t", "").strip()
+                if t in ("", "upload"):
+                    mutable = bool(get_arg(req, "mutable", "").strip())
+                    if mutable:
+                        return unlinked.UnlinkedPOSTSSKUploader(), ()
+                    else:
+                        return unlinked.UnlinkedPOSTCHKUploader(client, req), ()
+                if t == "mkdir":
+                    return unlinked.UnlinkedPOSTCreateDirectory(), ()
+                errmsg = "/uri accepts only PUT, PUT?t=mkdir, POST?t=upload, and POST?t=mkdir"
+                return WebError(http.BAD_REQUEST, errmsg), ()
+
+        if len(segments) < 2:
+            return rend.NotFound
+
+        uri = segments[1]
+        d = defer.maybeDeferred(client.create_node_from_uri, uri)
+        d.addCallback(lambda node: VDrive(node, uri))
+        d.addCallback(lambda vd: vd.locateChild(ctx, segments[2:]))
+        def _trap_KeyError(f):
+            f.trap(KeyError)
+            return rend.FourOhFour(), ()
+        d.addErrback(_trap_KeyError)
+        return d
 
     child_webform_css = webform.defaultCSS
     child_tahoe_css = nevow_File(resource_filename('allmydata.web', 'tahoe.css'))
